/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.file.nio;

import org.gradle.internal.file.FileCanonicalizer;
import org.gradle.internal.file.FileException;

import java.io.File;
import java.io.IOException;

public class Jdk7FileCanonicalizer implements FileCanonicalizer {
    @Override
    public File canonicalize(File file) throws FileException {
        try {
            return file.toPath().toRealPath().toFile();
        } catch (IOException e) {
            throw new FileException(String.format("Could not canonicalize file %s.", file), e);
        }
    }
}
