/*
 * Copyright 2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.tooling.events.test;

/**
 * Enumerates the different kinds of JVM tests. This allows to differentiate between test suites, atomic tests, etc.
 *
 * @since 2.4
 */
public enum JvmTestKind {

    SUITE("Test suite"),
    ATOMIC("Atomic test"),
    UNKNOWN("Unknown test kind");

    private final String label;

    JvmTestKind(String label) {
        this.label = label;
    }

    /**
     * Returns a label for the test kind. The label can be used to generate a prettified version of the test descriptor.
     *
     * @return a label corresponding to the test kind
     */
    public String getLabel() {
        return label;
    }

}
