/*
 * Copyright 2019 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.workers.internal;

import com.google.common.base.Objects;
import org.gradle.internal.classloader.VisitableURLClassLoader;

public class FlatClassLoaderStructure implements ClassLoaderStructure {
    private final VisitableURLClassLoader.Spec spec;

    public FlatClassLoaderStructure(VisitableURLClassLoader.Spec spec) {
        this.spec = spec;
    }

    @Override
    public VisitableURLClassLoader.Spec getSpec() {
        return spec;
    }

    @Override
    public ClassLoaderStructure getParent() {
        return null;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        FlatClassLoaderStructure that = (FlatClassLoaderStructure) o;
        return Objects.equal(spec, that.spec);
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(spec);
    }
}
