/*
 * Copyright 2016 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.nativeintegration.jansi;

import java.io.File;

public class JansiStorageLocator {

    private JansiLibraryFactory factory = new JansiLibraryFactory();

    void setFactory(JansiLibraryFactory factory) {
        this.factory = factory;
    }

    public JansiStorage locate(File storageDir) {
        JansiLibrary jansiLibrary = factory.create();

        if (jansiLibrary != null) {
            File jansiDir = makeVersionSpecificDir(storageDir);
            File targetLibFile = new File(jansiDir, jansiLibrary.getPath());
            return new JansiStorage(jansiLibrary, targetLibFile);
        }

        return null;
    }

    private File makeVersionSpecificDir(File storageDir) {
        String jansiVersion = org.fusesource.jansi.Ansi.class.getPackage().getImplementationVersion();
        return new File(storageDir, "jansi/" + jansiVersion);
    }
}
