/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api;

import org.jspecify.annotations.Nullable;

/**
 * <code>UncheckedIOException</code> is used to wrap an {@link java.io.IOException} into an unchecked exception.
 *
 * @deprecated Use java.io.UncheckedIOException instead. Will be removed in Gradle 10.
 */
@Deprecated
public class UncheckedIOException extends RuntimeException {
    public UncheckedIOException() {
    }

    public UncheckedIOException(@Nullable String message) {
        super(message);
    }

    public UncheckedIOException(@Nullable String message, @Nullable Throwable cause) {
        super(message, cause);
    }

    public UncheckedIOException(@Nullable Throwable cause) {
        super(cause);
    }
}
