/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.tasks.scala;

import org.gradle.api.internal.tasks.compile.JvmLanguageCompileSpec;

import java.io.File;
import java.util.Map;

public interface ScalaCompileSpec extends JvmLanguageCompileSpec {
    MinimalScalaCompileOptions getScalaCompileOptions();

    Iterable<File> getScalaCompilerPlugins();

    void setScalaCompilerPlugins(Iterable<File> plugins);

    File getClassfileBackupDir();

    void setClassfileBackupDir(File classfileBackupDir);

    File getAnalysisFile();

    void setAnalysisFile(File analysisFile);

    Map<File, File> getAnalysisMap();

    void setAnalysisMap(Map<File, File> analysisMap);

    long getBuildStartTimestamp();
}
