/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.process.internal;

import org.jspecify.annotations.NullMarked;
import org.jspecify.annotations.Nullable;

import java.io.OutputStream;

@NullMarked
public interface BaseExecHandleBuilder {

    BaseExecHandleBuilder setDisplayName(@Nullable String displayName);
    BaseExecHandleBuilder setStandardOutput(OutputStream outputStream);
    BaseExecHandleBuilder setErrorOutput(OutputStream outputStream);

    /**
     * Adds a listener to the list of ExecHandle listeners..
     */
    BaseExecHandleBuilder listener(ExecHandleListener listener);

    ExecHandle build();
}
