/*
 * Copyright 2020 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.plugin.devel.internal.precompiled;

import org.gradle.configuration.DefaultScriptTarget;

class PrecompiledScriptTarget extends DefaultScriptTarget {

    private final boolean supportsPluginsBlock;
    private final boolean supportsPluginManagementBlock;

    PrecompiledScriptTarget(boolean supportsPluginsBlock, boolean supportsPluginManagementBlock) {
        super(null);
        this.supportsPluginsBlock = supportsPluginsBlock;
        this.supportsPluginManagementBlock = supportsPluginManagementBlock;
    }

    @Override
    public boolean getSupportsPluginsBlock() {
        return supportsPluginsBlock;
    }

    @Override
    public boolean getSupportsPluginManagementBlock() {
        return supportsPluginManagementBlock;
    }

    @Override
    public void addConfiguration(Runnable runnable, boolean deferrable) {
        throw new UnsupportedOperationException();
    }
}
