/*
 * Copyright 2026 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.cli;

import org.jspecify.annotations.NullMarked;

/**
 * Categories for build options. Used in the help output.
 */
@NullMarked
public enum OptionCategory {
    HELP("Help"),
    LOGGING("Logging"),
    CONSOLE("Console"),
    CONFIGURATION("Configuration"),
    EXECUTION("Execution"),
    PERFORMANCE("Performance"),
    SECURITY("Security"),
    DIAGNOSTICS("Diagnostics"),
    DAEMON("Daemon"),
    DEVELOCITY("Develocity"),
    OTHER("");

    private final String displayName;

    OptionCategory(String displayName) {
        this.displayName = displayName;
    }

    public String getDisplayName() {
        return displayName;
    }
}
