/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.specs;

import org.gradle.api.specs.Spec;

/**
 * A predicate against objects of type T that can explain the unsatisfied reason.
 *
 * @param <T> The target type for this Spec
 */
public interface ExplainingSpec<T> extends Spec<T> {

    /**
     * Explains why the spec is not satisfied.
     *
     * @param element candidate
     * @return the description. Must not be null if the spec is not satisfied. Is null if spec is satisfied.
     */
    String whyUnsatisfied(T element);
}
