/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.nativeplatform.toolchain.internal.xcode;

import com.google.common.collect.ImmutableList;
import org.gradle.internal.service.scopes.Scope;
import org.gradle.internal.service.scopes.ServiceScope;
import org.gradle.process.internal.ExecActionFactory;

import javax.inject.Inject;
import java.util.List;

@ServiceScope(Scope.BuildSession.class)
public class MacOSSdkPlatformPathLocator extends AbstractLocator {
    @Inject
    public MacOSSdkPlatformPathLocator(ExecActionFactory execActionFactory) {
        super(execActionFactory);
    }

    @Override
    protected List<String> getXcrunFlags() {
        return ImmutableList.of("--sdk", "macosx", "--show-sdk-platform-path");
    }
}
