/*
 * Copyright 2025 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.collect;

import java.util.Iterator;

final class ToString {

    static <T> String nonEmptyIterator(Iterator<T> iterator) {
        return nonEmptyIterator('[', iterator, ']');
    }

    static <K> String nonEmptyIterator(char open, Iterator<K> iterator, char close) {
        StringBuilder string = new StringBuilder();
        string.append(open);
        string.append(iterator.next());
        while (iterator.hasNext()) {
            string.append(',');
            string.append(iterator.next());
        }
        return string.append(close).toString();
    }

    static <K, V> String entry(K key, V value) {
        return key + ":" + value;
    }
}
