/*
 * Copyright 2021 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.cc.impl.cacheentry

import org.gradle.cache.internal.streams.BlockAddress
import org.gradle.internal.serialize.graph.codecs.ValueObject
import org.gradle.util.Path
import java.io.File


/**
 * Data stored in the "entry details" file. Provides some metadata about the cache entry.
 */
internal
class EntryDetails(
    val buildInvocationScopeId: String,
    val rootDirs: List<File>,
    val intermediateModels: Map<ModelKey, BlockAddress>,
    val projectMetadata: Map<Path, BlockAddress>,
    val sideEffects: List<BlockAddress>,
) : ValueObject
