/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.launcher.daemon.client;

/**
 * Thrown when connected to a stale daemon address.
 *
 * This is thrown instead of using an initial handshake on the connection, to avoid the latency of a round trip to the
 * daemon before starting the build.
 */
public class StaleDaemonAddressException extends DaemonConnectionException {
    public StaleDaemonAddressException(String message, Throwable cause) {
        super(message, cause);
    }
}
