/*
 * Copyright 2016 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.reporting;

import org.gradle.api.resources.TextResource;
import org.gradle.api.tasks.Nested;
import org.gradle.api.tasks.Optional;
import org.jspecify.annotations.Nullable;

/**
 * A HTML Report whose generation can be customized with a XSLT stylesheet.
 */
public interface CustomizableHtmlReport extends SingleFileReport {

    /**
     * The stylesheet to use to generate the HTML report.
     *
     * @return the stylesheet to use to generate the HTML report
     */
    @Nullable
    @Optional
    @Nested
    TextResource getStylesheet();

    /**
     * The stylesheet to use to generate the report.
     *
     * @param stylesheet the stylesheet to use to generate the HTML report
     */
    void setStylesheet(@Nullable TextResource stylesheet);

}
