/*
 * Copyright 2021 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.tasks.compile.incremental.compilerapi.constants;

import org.gradle.api.internal.tasks.compile.incremental.compilerapi.deps.DependentsSet;

import java.util.Collections;
import java.util.Map;

public class ConstantToDependentsMapping {

    private final Map<String, DependentsSet> constantDependents;

    public ConstantToDependentsMapping(Map<String, DependentsSet> constantDependents) {
        this.constantDependents = constantDependents;
    }

    public Map<String, DependentsSet> getConstantDependents() {
        return constantDependents;
    }

    public DependentsSet getConstantDependentsForClass(String constantOrigin) {
        return constantDependents.getOrDefault(constantOrigin, DependentsSet.empty());
    }

    public static ConstantToDependentsMapping empty() {
        return new ConstantToDependentsMapping(Collections.emptyMap());
    }

    public static ConstantToDependentsMappingBuilder builder() {
        return new ConstantToDependentsMappingBuilder();
    }

}
