/*
 * Copyright 2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.taskgraph;

/**
 * The identity of a node in the execution graph.
 * <p>
 * Needs to be unique across all the execution graphs of the current build.
 *
 * @implNote The implementations must provide value-based equality.
 * @since 8.1
 */
public interface NodeIdentity {

    enum NodeType {
        TASK,
        TRANSFORM_STEP
    }

    NodeType getNodeType();

}
