/*
 * Copyright 2021 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.util.internal;

import java.io.IOException;
import java.io.InputStream;

public abstract class BulkReadInputStream extends InputStream {
    @Override
    public int read() throws IOException {
        byte[] buffer = new byte[1];
        while (true) {
            int nread = read(buffer);
            if (nread < 0) {
                return -1;
            }
            if (nread == 1) {
                return 0xff & buffer[0];
            }
        }
    }

    @Override
    public abstract int read(byte[] bytes, int pos, int count) throws IOException;
}
