/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.component.resolution.failure.exception;

import org.gradle.internal.component.resolution.failure.interfaces.VariantSelectionByNameFailure;

import java.util.List;

/**
 * Represents a failure during variant selection when a variant cannot be selected via an explicit request for a variant by name.
 */
public final class VariantSelectionByNameException extends AbstractResolutionFailureException {
    public VariantSelectionByNameException(String message, VariantSelectionByNameFailure failure, List<String> resolutions) {
        super(message, failure, resolutions);
    }

    @Override
    public VariantSelectionByNameFailure getFailure() {
        return (VariantSelectionByNameFailure) failure;
    }
}
