/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.publish.ivy.internal.dependency;

import org.gradle.api.artifacts.ExcludeRule;

public class DefaultIvyExcludeRule implements IvyExcludeRule {

    private final ExcludeRule excludeRule;
    private final String ivyConfiguration;

    public DefaultIvyExcludeRule(ExcludeRule excludeRule, String ivyConfiguration) {
        this.excludeRule = excludeRule;
        this.ivyConfiguration = ivyConfiguration;
    }

    @Override
    public String getOrg() {
        return excludeRule.getGroup();
    }

    @Override
    public String getModule() {
        return excludeRule.getModule();
    }

    @Override
    public String getConf() {
        return ivyConfiguration;
    }
}
