/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.rules;

import org.gradle.api.specs.Spec;

/**
 * Represents a tuple containing a Spec and a RuleAction
 */
public class SpecRuleAction<T> {
    final RuleAction<? super T> action;
    final Spec<? super T> spec;

    public SpecRuleAction(RuleAction<? super T> action, Spec<? super T> spec) {
        this.action = action;
        this.spec = spec;
    }

    public RuleAction<? super T> getAction() {
        return action;
    }

    public Spec<? super T> getSpec() {
        return spec;
    }
}
