/*
 * Copyright 2026 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.tooling.internal.consumer.converters;

import org.jspecify.annotations.NullMarked;

import java.io.Serializable;

@NullMarked
public class BuildEnvironmentVersionInfoMixin implements Serializable {
    private final String version;

    public BuildEnvironmentVersionInfoMixin(String version) {
        this.version = version;
    }

    @SuppressWarnings("unused")// Used by reflection
    public String getVersionInfo() {
        return String.join(System.lineSeparator(),
            "",
            "------------------------------------------------------------",
            "Gradle " + version,
            "------------------------------------------------------------",
            "");
    }
}
