/*
 * Copyright 2025 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.collections;

import org.gradle.api.internal.CollectionCallbackActionDecorator;
import org.gradle.api.internal.DefaultCollectionCallbackActionDecorator;
import org.gradle.api.internal.MutationGuards;
import org.gradle.internal.code.UserCodeApplicationContext;
import org.gradle.internal.instantiation.InstantiatorFactory;
import org.gradle.internal.operations.BuildOperationRunner;
import org.gradle.internal.service.Provides;
import org.gradle.internal.service.ServiceRegistration;
import org.gradle.internal.service.ServiceRegistrationProvider;
import org.gradle.internal.service.ServiceRegistry;
import org.gradle.internal.service.scopes.AbstractGradleModuleServices;

public final class DomainObjectCollectionServices extends AbstractGradleModuleServices {
    @Override
    public void registerCrossBuildSessionServices(ServiceRegistration registration) {
        registration.addProvider(new ServiceRegistrationProvider() {
            @Provides
            CollectionCallbackActionDecorator createDomainObjectCollectionCallbackActionDecorator(
                BuildOperationRunner buildOperationRunner,
                UserCodeApplicationContext userCodeApplicationContext
            ) {
                return new DefaultCollectionCallbackActionDecorator(buildOperationRunner, userCodeApplicationContext);
            }
        });
    }

    @Override
    public void registerGlobalServices(ServiceRegistration registration) {
        registration.addProvider(new ServiceRegistrationProvider() {
            @Provides
            DomainObjectCollectionFactory createDomainObjectCollectionFactory(
                InstantiatorFactory instantiatorFactory,
                ServiceRegistry services
            ) {
                return new DefaultDomainObjectCollectionFactory(instantiatorFactory, services, CollectionCallbackActionDecorator.NOOP, MutationGuards.identity());
            }
        });
    }
}
