/*
 * Copyright 2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.operations.logging;

import java.io.File;

class BuildOperationLogInfo {
    private final File outputFile;
    private final int maximumFailures;
    private final String taskName;

    BuildOperationLogInfo(String taskName, File outputFile, int maximumFailures) {
        this.outputFile = outputFile;
        this.maximumFailures = maximumFailures;
        this.taskName = taskName;
    }

    public int getMaximumFailedOperationsShown() {
        return maximumFailures;
    }

    public File getOutputFile() {
        return outputFile;
    }

    public String getTaskName() {
        return taskName;
    }

    @Override
    public String toString() {
        return "log for " + getTaskName() + " to " + getOutputFile().getAbsolutePath() + ", showing up to " + getMaximumFailedOperationsShown() + " failures";
    }
}
