/*
 * Copyright 2016 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api.internal.attributes;

public interface DisambiguationRule<T> {

    /* private */ DisambiguationRule<Object> DO_NOTHING = new DisambiguationRule<Object>() {
        @Override
        public boolean doesSomething() {
            return false;
        }

        @Override
        public void execute(MultipleCandidatesResult<Object> t) { }
    };

    @SuppressWarnings("unchecked")
    static <E> DisambiguationRule<E> doNothing() {
        return (DisambiguationRule<E>) DO_NOTHING;
    }

    boolean doesSomething();

    void execute(MultipleCandidatesResult<T> t);
}
