/*
 * Copyright 2016 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.nativeintegration.jansi;

import org.fusesource.jansi.internal.OSInfo;

public class JansiLibraryFactory {
    public JansiLibrary create() {
        // From https://github.com/fusesource/jansi/blob/3d2a9788fa48e4cecbbe28279d01111a125d2f66/src/main/java/org/fusesource/jansi/internal/JansiLoader.java#L294-L298
        String jansiNativeLibraryName = System.mapLibraryName("jansi");
        if (jansiNativeLibraryName.endsWith(".dylib")) {
            jansiNativeLibraryName = jansiNativeLibraryName.replace(".dylib", ".jnilib");
        }
        return new JansiLibrary(OSInfo.getNativeLibFolderPathForCurrentOS(), jansiNativeLibraryName);
    }
}
