/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.nativeintegration.console;

public enum FallbackConsoleMetaData implements ConsoleMetaData {
    ATTACHED(true),
    NOT_ATTACHED(false);

    private final boolean attached;

    FallbackConsoleMetaData(boolean attached) {
        this.attached = attached;
    }

    @Override
    public boolean isStdOutATerminal() {
        return attached;
    }

    @Override
    public boolean isStdErrATerminal() {
        return attached;
    }

    @Override
    public int getCols() {
        return 0;
    }

    @Override
    public int getRows() {
        return 0;
    }

    @Override
    public boolean isWrapStreams() {
        return attached;
    }
}
