/*
 * Copyright 2026 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.cc.impl

import org.gradle.api.internal.GradleInternal
import org.gradle.api.internal.project.CrossBuildModelAccess
import org.gradle.internal.buildoption.InternalOptions
import org.gradle.internal.configuration.problems.IsolatedProjectsProblemsReporter

private val CROSS_BUILD_ACCESS_FLAG = InternalOptions.ofBoolean("org.gradle.internal.isolated-projects.report-cross-build-access", false)

internal
class ProblemReportingCrossBuildModelAccess(
    private val ipProblems: IsolatedProjectsProblemsReporter,
    private val internalOptions: InternalOptions
) : CrossBuildModelAccess {

    override fun access(referrer: GradleInternal, gradle: GradleInternal): GradleInternal =
        if (internalOptions.getBoolean(CROSS_BUILD_ACCESS_FLAG)) {
            CrossBuildConfigurationReportingGradle(gradle, referrer, ipProblems)
        } else {
            gradle
        }
}
