/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.initialization.definition;

/**
 * The DSL for declaring plugins to inject into an included build.
 *
 * @since 4.6
 */
public interface InjectedPluginDependencies {
    /**
     * Add a dependency on the plugin with the given id.
     *
     * @param id the id of the plugin to depend on
     *
     * @return a mutable injected plugin dependency that can be used to further refine the dependency
     */
    InjectedPluginDependency id(String id);
}
