/*
 * Copyright 2026 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.tasks.testing.report.generic;

import org.gradle.api.GradleException;
import org.gradle.internal.exceptions.ResolutionProvider;

import java.util.List;

public final class UnshrinkableReportPathException extends GradleException implements ResolutionProvider {
    public UnshrinkableReportPathException(String unshrinkablePath) {
        // Note: this message should not say something like "below the filesystem limit" because we use a potentially smaller limit.
        // See SafeFileLocationUtils.MAX_PATH_LENGTH.
        super("Cannot shrink report path below required limit. Path that could not be shrunk (relative to the report directory): " + unshrinkablePath);
    }

    @Override
    public List<String> getResolutions() {
        return List.of(
            "Use a shorter report directory path.",
            "Reduce nesting in your tests.",
            "Disable the HTML report for this task."
        );
    }
}
