/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.cache.internal;

import org.gradle.cache.CacheCleanupStrategy;
import org.gradle.cache.CacheCleanupStrategyFactory;
import org.gradle.cache.CleanupAction;
import org.gradle.cache.CleanupFrequency;
import org.gradle.internal.operations.BuildOperationRunner;

import java.util.function.Supplier;

public class DefaultCacheCleanupStrategyFactory implements CacheCleanupStrategyFactory {
    private final BuildOperationRunner buildOperationRunner;

    public DefaultCacheCleanupStrategyFactory(BuildOperationRunner buildOperationRunner) {
        this.buildOperationRunner = buildOperationRunner;
    }

    @Override
    public CacheCleanupStrategy daily(CleanupAction action) {
        return new DefaultCacheCleanupStrategy(action, () -> CleanupFrequency.DAILY, buildOperationRunner);
    }

    @Override
    public CacheCleanupStrategy create(CleanupAction action, Supplier<CleanupFrequency> frequency) {
        return new DefaultCacheCleanupStrategy(action, frequency, buildOperationRunner);
    }
}
