/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api.tasks;

/**
 * Provides information about some work which was performed.
 */
public interface WorkResult {
    boolean getDidWork();

    /**
     * Returns this result if it did work, otherwise returns the result given as a parameter.
     *
     * @since 6.0
     */
    default WorkResult or(WorkResult other) {
        if (!getDidWork()) {
            return other;
        } else {
            return this;
        }
    }
}
