/*
 * Copyright 2016 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.reporting.internal;

import org.gradle.api.Describable;
import org.gradle.api.reporting.CustomizableHtmlReport;
import org.gradle.api.resources.TextResource;
import org.jspecify.annotations.Nullable;

public abstract class CustomizableHtmlReportImpl extends DefaultSingleFileReport implements CustomizableHtmlReport {

    private TextResource stylesheet;

    public CustomizableHtmlReportImpl(String name, Describable owner) {
        super(name, owner);
    }

    @Override
    @Nullable
    public TextResource getStylesheet() {
        return stylesheet;
    }

    @Override
    public void setStylesheet(@Nullable TextResource stylesheet) {
        this.stylesheet = stylesheet;
    }

}
