/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.initialization;

import org.gradle.api.artifacts.dsl.DependencyHandler;
import org.gradle.api.internal.initialization.transform.services.CacheInstrumentationDataBuildService;
import org.gradle.api.provider.Provider;

public class ScriptClassPathResolutionContext {
    private final long contextId;
    private final Provider<CacheInstrumentationDataBuildService> buildService;
    private final DependencyHandler dependencyHandler;

    public ScriptClassPathResolutionContext(
        long contextId,
        Provider<CacheInstrumentationDataBuildService> buildService,
        DependencyHandler dependencyHandler
    ) {
        this.contextId = contextId;
        this.buildService = buildService;
        this.dependencyHandler = dependencyHandler;
    }

    public long getContextId() {
        return contextId;
    }

    public Provider<CacheInstrumentationDataBuildService> getBuildService() {
        return buildService;
    }

    public DependencyHandler getDependencyHandler() {
        return dependencyHandler;
    }
}
