/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.model.internal.core.rule.describe;

import com.google.errorprone.annotations.FormatMethod;
import com.google.errorprone.annotations.FormatString;

/**
 * Describes a method rule.
 * All implementations of this class are expected to implement the equals and hashCode method
 */
public interface ModelRuleDescriptor {
    /**
     * This method is expected to be idempotent.
     *
     * @param appendable where to write the description to.
     */
    void describeTo(Appendable appendable);

    ModelRuleDescriptor append(ModelRuleDescriptor child);

    ModelRuleDescriptor append(String child);

    @FormatMethod
    ModelRuleDescriptor append(@FormatString String child, Object... args);
}
