/*
 * Copyright 2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.component;

import org.gradle.api.component.SoftwareComponent;
import org.gradle.api.component.SoftwareComponentContainer;
import org.gradle.api.provider.Property;

/**
 * Internal counterpart of {@link SoftwareComponentContainer}.
 */
public interface SoftwareComponentContainerInternal extends SoftwareComponentContainer {
    /**
     * Get the main component of a project.
     * <p>
     * This method will remain internal until we pinpoint what a "main" component truly is,
     * and if it is conceptually necessary.
     */
    Property<SoftwareComponent> getMainComponent();
}
