/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.buildconfiguration.tasks;

import org.gradle.internal.buildconfiguration.DaemonJvmPropertiesDefaults;
import org.gradle.platform.Architecture;
import org.gradle.platform.BuildPlatform;
import org.gradle.platform.BuildPlatformFactory;
import org.gradle.platform.OperatingSystem;

public class DaemonJvmPropertiesUtils {

    public static String getToolchainUrlPropertyForPlatform(BuildPlatform buildPlatform) {
        String operatingSystemName = buildPlatform.getOperatingSystem().name();
        String architectureName = buildPlatform.getArchitecture().name();
        return String.format(DaemonJvmPropertiesDefaults.TOOLCHAIN_URL_PROPERTY_FORMAT, operatingSystemName, architectureName);
    }

    public static BuildPlatform getPlatformFromToolchainProperty(String toolchainUrlProperty) {
        String[] parts = toolchainUrlProperty.split("\\.");
        if (parts.length != 3) {
            throw new IllegalArgumentException(String.format("Invalid toolchain URL property name: %s", toolchainUrlProperty));
        }
        return BuildPlatformFactory.of(Architecture.valueOf(parts[2]), OperatingSystem.valueOf(parts[1]));
    }
}
