/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api.internal.project;

import org.gradle.api.Project;
import org.gradle.api.artifacts.component.BuildIdentifier;
import org.gradle.api.artifacts.component.ProjectComponentIdentifier;
import org.gradle.initialization.ProjectDescriptorRegistry;
import org.gradle.internal.Factory;
import org.gradle.internal.build.BuildProjectRegistry;
import org.gradle.internal.build.BuildState;
import org.gradle.internal.project.ImmutableProjectDescriptor;
import org.gradle.internal.service.scopes.Scope;
import org.gradle.internal.service.scopes.ServiceScope;
import org.gradle.util.Path;
import org.jspecify.annotations.Nullable;

import javax.annotation.concurrent.ThreadSafe;

/**
 * A registry of all projects present in a build tree.
 */
@ThreadSafe
@ServiceScope(Scope.BuildTree.class)
public interface ProjectStateRegistry extends ProjectStateLookup {
    /**
     * Locates the state object that owns the given public project model. Can use {@link ProjectInternal#getOwner()} instead.
     */
    ProjectState stateFor(Project project) throws IllegalArgumentException;

    /**
     * Locates the state object that owns the project with the given identifier.
     */
    ProjectState stateFor(ProjectComponentIdentifier identifier) throws IllegalArgumentException;

    /**
     * Locates the state object that owns the project with the identity path.
     */
    ProjectState stateFor(Path identityPath) throws IllegalArgumentException;

    /**
     * Locates the state objects for all projects of the given build.
     */
    BuildProjectRegistry projectsFor(BuildIdentifier buildIdentifier) throws IllegalArgumentException;

    /**
     * Locates the state objects for all projects of the given build, or {@code null} if these are not available yet.
     */
    @Nullable
    BuildProjectRegistry findProjectsFor(BuildIdentifier buildIdentifier);

    /**
     * Registers the projects of a build.
     */
    void registerProjects(BuildState owner, ProjectDescriptorRegistry projectRegistry);

    /**
     * Registers a single project.
     */
    ProjectState registerProject(BuildState owner, ImmutableProjectDescriptor projectDescriptor);

    /**
     * Allows the given code to access the mutable state of any project in the tree, regardless of which other threads may be accessing the project.
     *
     * DO NOT USE THIS METHOD. It is here to allow some very specific backwards compatibility.
     */
    <T> T allowUncontrolledAccessToAnyProject(Factory<T> factory);

    /**
     * Discards all projects for the given build.
     */
    void discardProjectsFor(BuildState build);
}
