/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.resource.local;

import org.gradle.internal.Factory;
import org.gradle.internal.hash.ChecksumService;

import java.io.File;
import java.util.List;
import java.util.function.Function;

public class AbstractLocallyAvailableResourceFinder<C> implements LocallyAvailableResourceFinder<C> {

    private final Function<C, Factory<List<File>>> producer;
    private final ChecksumService checksumService;

    public AbstractLocallyAvailableResourceFinder(Function<C, Factory<List<File>>> producer, ChecksumService checksumService) {
        this.producer = producer;
        this.checksumService = checksumService;
    }

    @Override
    public LocallyAvailableResourceCandidates findCandidates(C criterion) {
        return new LazyLocallyAvailableResourceCandidates(producer.apply(criterion), checksumService);
    }

    public ChecksumService getChecksumService() {
        return checksumService;
    }
}
